macroscript SplineTangentAligner
category:"BodyulCG"
tooltip:"Spline Tangent Aligner"
(
    local dialogPos = unsupplied
    
    rollout SplineTangentAligner "Spl. Tangent Aligner" width:170
    (
        group "Align"
        (
            button btnAlignIn "IN" width:45 align:#center across:3
            button btnAlignBoth "BOTH" width:45 align:#center
            button btnAlignOut "OUT" width:45 align:#center
        )
        
        fn isSplineShape shape =
        (
            local isNode = isValidNode shape
            local isShape = isNode and isShapeObject shape
            local isSplineShape = isShape and classof shape.baseobject == SplineShape
            local isLine = isShape and classof shape.baseobject == Line
            
            return isSplineShape or isLine
        )
        
        fn alignTangent shape spline knot tangent =
        (
            if isSplineShape shape and \
                numSplines shape >= spline and \
                numKnots shape spline >= knot then
            (
                local inVec = getInVec shape spline knot
                local knotPos = getKnotPoint shape spline knot
                local outVec = getOutVec shape spline knot
                
                case tangent of
                (
                    #in: (
                        setInVec shape spline knot \
                            (knotPos + normalize (knotPos - outVec) * (distance inVec knotPos))
                    )
                    #out: (
                        setOutVec shape spline knot \
                            (knotPos + normalize (knotPos - inVec) * (distance outVec knotPos))
                    )
                    #both: (
                        setInVec shape spline knot \
                            (knotPos + normalize (knotPos - outVec + inVec - knotPos) * (distance inVec knotPos))
                                
                        setOutVec shape spline knot \
                            (knotPos + normalize (knotPos - inVec + outVec - knotPos) * (distance outVec knotPos))
                    )
                )
                updateshape shape 
            )
        )
            
        fn doAlign shape tangent =
        (
            if isSplineShape shape and numKnots shape > 0 then
            (
                for s = 1 to numSplines shape do
                    for k in getKnotSelection shape s do
                    (
                        alignTangent shape s k tangent
                    )
            )
        )
        
        fn draw =
        (
            gw.setTransform (matrix3 1)
                
            if filters.Is_EditSpline() and subobjectlevel == 1 then
            (
                local shp = $
                for s = 1 to numSplines shp do
                (
                    local knots = getKnotSelection shp s
                    for k in knots where getKnotType shp s k == #bezierCorner do
                    (
                        local kpos = getKnotPoint shp s k
                        gw.wtext (gw.wTransPoint (getInVec shp s k) + [5, -5, 0]) "IN" color:yellow
                        gw.wtext (gw.wTransPoint (getOutVec shp s k) + [5, -5, 0]) "OUT" color:yellow
                    )
                )
            )
            
            gw.enlargeUpdateRect #whole
            gw.updateScreen()
        )
        
        on SplineTangentAligner open do
        (
            registerRedrawViewsCallback draw
            redrawViews()
        )
        
        on btnAlignIn pressed do doAlign $ #in
        on btnAlignBoth pressed do doAlign $ #both
        on btnAlignOut pressed do doAlign $ #out

        on SplineTangentAligner close do
        (
            unregisterRedrawViewsCallback draw
            redrawViews()
            dialogPos = GetDialogPos SplineTangentAligner
        )
    )
    
    on execute do
        if SplineTangentAligner.open then
            DestroyDialog SplineTangentAligner
        else
        (
            CreateDialog SplineTangentAligner pos:dialogPos
        )
)
(dotnetclass "MessageBox").Show "Spline Tangent Aligner installed\n\nGroup: Main UI\nCategory: BodyulCG" "Spline Tangent Aligner" \
    (dotnetclass "MessageBoxButtons").Ok \
    (dotnetclass "MessageBoxIcon").Information;